@@ -10,7 +10,7 @@ my $json = JSON->new->allow_nonref;
 
 use parent qw(Perinci::Access::Schemeless);
 
-our $VERSION = '0.03'; # VERSION
+our $VERSION = '0.04'; # VERSION
 
 sub new {
     my $class = shift;
@@ -19,6 +19,8 @@ sub new {
     # check required attributes
     die "Please specify required attribute 'dbh'" unless $self->{dbh};
 
+    $self->{fallback_on_completion} //= 0;
+
     $self;
 }
 
@@ -34,7 +36,8 @@ sub get_meta {
         if ($meta) {
             $req->{-meta} = $json->decode($meta);
         } else {
-            return [404, "No metadata found in database"];
+            return [404, "No metadata found in database for module ".
+                        "'$req->{-perl_package}' and function '$leaf'"];
         }
     } else {
         # XXP check in database, if exists return if not return {v=>1.1}
@@ -61,11 +64,11 @@ sub action_list {
     my $filter_path = sub {
         my $path = shift;
         if (defined($self->{allow_paths}) &&
-                !__match_paths2($path, $self->{allow_paths})) {
+                !Perinci::Access::Schemeless::__match_paths2($path, $self->{allow_paths})) {
             return 0;
         }
         if (defined($self->{deny_paths}) &&
-                __match_paths2($path, $self->{deny_paths})) {
+                Perinci::Access::Schemeless::__match_paths2($path, $self->{deny_paths})) {
             return 0;
         }
         1;
@@ -133,6 +136,44 @@ sub action_list {
     [200, "OK (list action)", \@res];
 }
 
+sub action_complete_arg_val {
+    my ($self, $req) = @_;
+
+    goto FALLBACK unless $self->{fallback_on_completion};
+
+    my $arg = $req->{arg} or return err(400, "Please specify arg");
+
+    $self->get_meta($req);
+    my $c = $req->{-meta}{args}{$arg}{completion};
+    goto FALLBACK unless defined($c) && ref($c) ne 'CODE';
+
+    # get meta from parent's get_meta
+    no warnings 'redefine';
+    local *get_meta = \&Perinci::Access::Schemeless::get_meta;
+    delete $req->{-meta};
+
+  FALLBACK:
+    $self->SUPER::action_complete_arg_val($req);
+}
+
+sub action_complete_arg_elem {
+    my ($self, $req) = @_;
+
+    goto FALLBACK unless $self->{fallback_on_completion};
+
+    my $arg = $req->{arg} or return err(400, "Please specify arg");
+
+    my $c = $req->{-meta}{$arg}{element_completion};
+    goto FALLBACK unless defined($c) && ref($c) ne 'CODE';
+
+    # get meta from parent's get_meta
+    local *get_meta = \&Perinci::Access::Schemeless::get_meta;
+    delete $req->{-meta};
+
+  FALLBACK:
+    $self->SUPER::action_complete_arg_elem($req);
+}
+
 1;
 # ABSTRACT: Subclass of Perinci::Access::Schemeless which gets lists of entities (and metadata) from DBI database
 
@@ -148,7 +189,7 @@ Perinci::Access::Schemeless::DBI - Subclass of Perinci::Access::Schemeless which
 
 =head1 VERSION
 
-version 0.03
+This document describes version 0.04 of Perinci::Access::Schemeless::DBI (from Perl distribution Perinci-Access-Schemeless-DBI), released on 2014-06-20.
 
 =head1 SYNOPSIS
 
@@ -213,6 +254,13 @@ Aside from its parent class, this class recognizes these attributes:
 
 DBI database handle.
 
+=item * fallback_on_completion => BOOL (default: 0)
+
+If set to true, then for C<complete_arg_val> and C<complete_arg_elem>, if
+metadata has a non-coderef C<completion> or C<element_completion> in its
+argument spec, then will fallback to parent class L<Perinci::Access::Schemeless>
+for metadata.
+
 =back
 
 =head1 FAQ
@@ -222,6 +270,36 @@ DBI database handle.
 If you have a large number of packages and functions, you might want to avoid
 reading Perl modules on the filesystem.
 
+=head2 I have completion routine for my argument, completion no longer works?
+
+For example, suppose your function metadata is something like this:
+
+ {
+     v => 1.1,
+     summary => 'Delete account',
+     args => {
+         name => {
+             summary => 'Account name',
+             completion => sub {
+                 my %args = @_;
+                 my $word = $args{word};
+                 search_accounts(prefix => $word);
+             },
+         },
+     },
+ }
+
+When this is stored in the database, most serialization format (JSON included)
+doesn't save the code in C<completion>. If you use L<Data::Clean::JSON>, by
+default the coderef will be replaced with plain string C<CODE>. This prevents
+completion to work e.g. if you request with this Riap request:
+
+ {action=>'complete_arg_val', uri=>..., arg=>'name'}
+
+One solution is to fallback to its parent class L<Perinci::Access::Schemeless>
+(which reads metadata from Perl source files) for meta request when doing
+completion. To do this, you can set the attribute C<fallback_on_completion>.
+
 =head1 TODO
 
 =over
